#if defined(SHARE_DEBUG)
#define LOG_UNGETTY
#define LOG_HDBDIAL
#endif

/* #define CHOOSE_DEBUG */

/*+-------------------------------------------------------------------------
	hdbintf.c - HDB UUCP database and /etc/utmp interface routines
	wht@n4hgf.Mt-Park.GA.US

  Defined functions:
	choose_device(type,baud)
	choose_line(baud)
	dialcodes_translate(phone)
	dialstr_translate(translate_list,to_translate)
	dvtype_match(typespec,dvtype)
	enddlent()
	enddvent()
	getdlent()
	getdlentname(name)
	getdvbaud(baud)
	getdvent()
	getdvline(line)
	getdvtype(type)
	hdb_dial(presult)
	hdb_dial_error_text(errcode)
	hdb_init()
	strip_phone_num(sptr)
	ungetty_get_line()
	ungetty_return_line()

Date: Fri, 23 Aug 91 18:30:06 +0300 (MSD)
From: emory!hq.demos.su!ache (Andrew A. Chernov, canton Uri's citizen)
1) HDB dialers may return connect speed as return code (!= 0)
2) Using HDB Dialcodes file for phone numbers translation now
   (\D,\T escape sequence)

--------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:09-01-1991-16:20-wht@n4hgf2-generalize HDB configuration files location */
/*:09-01-1991-02:27-wht@n4hgf2-dialer gets file name instead of "ECUdial" */
/*:08-25-1991-13:07-wht@n4hgf-apply ache@hq.demos.su patches */
/*:08-10-1991-17:39-wht@n4hgf-US_WEGOTIT handling */
/*:07-25-1991-12:58-wht@n4hgf-ECU release 3.10 */
/*:06-02-1991-17:42-wht@n4hgf-add getdvtype */
/*:06-02-1991-17:27-wht@n4hgf-add choose_device plus move choose_line here */
/*:10-16-1990-20:43-wht@n4hgf-add SHARE_DEBUG */
/*:09-19-1990-19:36-wht@n4hgf-ecu_log_event now gets pid for log from caller */
/*:08-14-1990-20:40-wht@n4hgf-ecu3.00-flush old edit history */

#include "ecu.h"
#include "esd.h"
#include "var.h"
#include "termecu.h"
#include "utmpstatus.h"
#include "ecuungetty.h"
#include "dvent.h"
#include "dlent.h"
#include "dialprog.h"
#include <errno.h>
#include <utmp.h>

char *arg_token();
char *skip_ld_break();
char *dialcodes_translate();
char *strip_phone_num();

extern char kbdintr;		/* current input INTR */
extern ulong colors_current;
extern char *sys_errlist[];
extern int proc_level;
extern int proctrace;

int there_is_hdb_on_this_machine = 0;
FILE *fpdv = NULL;
FILE *fpdl = NULL;
char *Devices_file = "/usr/lib/uucp/Devices";
char *Dialers_file = "/usr/lib/uucp/Dialers";
char *Dialcodes_file = "/usr/lib/uucp/Dialcodes";
char ungetty_ttyname[64] = "";

/*+-------------------------------------------------------------------------
	ungetty_get_line()
--------------------------------------------------------------------------*/
int
ungetty_get_line()
{
int itmp;
int ungetty_pid;
SIGTYPE (*original_sighdlr)();
int wait_status;
char ungetty[128];
#if defined(LOG_UNGETTY)
char ungetty_log[80];
#endif
char bamboozlement[20];
char *bamboozle();

	if(!there_is_hdb_on_this_machine)
		return(LOPEN_ENABLED);

	sprintf(ungetty,"%s/ecuungetty",ECULIBDIR);
	strcpy(bamboozlement,bamboozle(getpid()));
	if(access(ungetty,1))
	{
		pperror(ungetty);
		return(LOPEN_ENABLED);
	}
	original_sighdlr = signal(SIGCLD,SIG_DFL);
	if((ungetty_pid = smart_fork()) == 0)
	{
		execl(ungetty,"ungetty",shm->Lline,bamboozlement,(char *)0);
		exit(-1);
	}
	while(((itmp = wait(&wait_status)) != ungetty_pid) && (itmp != -1))
		;
	signal(SIGCLD,original_sighdlr);
	switch((wait_status >> 8) & 0xFF)
	{
		case UG_NOTENAB:	/* line acquired: not enabled */
			ungetty_ttyname[0] = 0;
			break;

		case UG_ENAB:		/* line acquired: need ungetty -r when done */
#if defined(LOG_UNGETTY)
			sprintf(ungetty_log,"UNGETTY acquired %s",shm->Lline);
			ecu_log_event(getpid(),ungetty_log);
#endif
			strcpy(ungetty_ttyname,shm->Lline);
			break;

		case UG_FAIL:		/* line in use */
			ungetty_ttyname[0] = 0;
			return(LOPEN_ENABLED_IN_USE);

		default:
#if defined(LOG_UNGETTY)
			sprintf(ungetty_log,"UNGETTY status 0x%04x",wait_status);
			ecu_log_event(getpid(),ungetty_log);
#endif
			ungetty_ttyname[0] = 0;
			return(LOPEN_ENABLED);
	}

	return(0);

}	/* end of ungetty_get_line */

/*+-------------------------------------------------------------------------
	ungetty_return_line()
--------------------------------------------------------------------------*/
void
ungetty_return_line()
{
int ungetty_pid;
int itmp;
SIGTYPE (*original_sighdlr)();
int wait_status = 0xDEAD;
char ungetty[128];
#if defined(LOG_UNGETTY)
char ungetty_log[80];
#endif
char bamboozlement[20];
char *bamboozle();

	if(!there_is_hdb_on_this_machine)
		return;
	if(!ungetty_ttyname[0])
		return;

	strcpy(bamboozlement,bamboozle(getpid()));

	/* call ungetty to see if we need to switch to dialin */
	sprintf(ungetty,"%s/ecuungetty",ECULIBDIR);
	strcpy(bamboozlement,bamboozle(getpid()));
	if(access(ungetty,1))
	{
		pperror(ungetty);
		return;
	}
	original_sighdlr = signal(SIGCLD,SIG_DFL);
	if((ungetty_pid = smart_fork()) == 0)
	{
		execl(ungetty,"ungetty","-t",ungetty_ttyname,bamboozlement,(char *)0);
		ecu_log_event(getpid(),"could not exec ecuungetty -t");
		exit(-1);
	}
	while(((itmp = wait(&wait_status)) != ungetty_pid) &&
			(itmp != -1) )
		;
	signal(SIGCLD,original_sighdlr);
	switch((wait_status >> 8) & 0xFF)
	{
		case UG_RESTART:
			break;

		default:
#if defined(LOG_UNGETTY)
			sprintf(ungetty_log,"UNGETTY -t %s status %04x",
				ungetty_ttyname,wait_status);
			ecu_log_event(getpid(),ungetty_log);
#endif
			ungetty_ttyname[0] = 0;
			return;
	}

	strcpy(bamboozlement,bamboozle(getpid()));

	original_sighdlr = signal(SIGCLD,SIG_DFL);
	if((ungetty_pid = smart_fork()) == 0)
	{
		execl(ungetty,"ungetty","-r",ungetty_ttyname,bamboozlement,(char *)0);
		ecu_log_event(getpid(),"could not exec ecuungetty -r");
		exit(-1);
	}

	while(((itmp = wait(&wait_status)) != ungetty_pid) &&
			(itmp != -1))
		;

#if defined(LOG_UNGETTY)
	if(wait_status)
	{
		sprintf(ungetty_log,"UNGETTY -r %s status 0x%04x",
			ungetty_ttyname,wait_status);
	}
	else
		sprintf(ungetty_log,"UNGETTY returned %s",ungetty_ttyname);
	ecu_log_event(getpid(),ungetty_log);
#endif

	ungetty_ttyname[0] = 0;

}	/* end of ungetty_return_line */

/*+-------------------------------------------------------------------------
	getdvent() - get first or next Devices entry (a la getpwent)
--------------------------------------------------------------------------*/
struct dvent *
getdvent()
{
int itmp;
char *cptr;
#define MAX_DV_TOKENS 9
char *tokens[MAX_DV_TOKENS];
static struct dvent dve;
static char dvstr[128];
char *strchr();

	if(!there_is_hdb_on_this_machine)
		return((struct dvent *)0);

	if(fpdv == NULL)
	{
		if((fpdv = fopen(Devices_file,"r")) == NULL)
		{
			pperror(Devices_file);
			return((struct dvent *)0);
		}
	}

	while(1)
	{
		if(fgets(dvstr,sizeof(dvstr),fpdv) == NULL)
			return((struct dvent *)0);

		if((dvstr[0] == '#') || (dvstr[0] == ' ') || (strlen(dvstr) == 1))
			continue;

		build_arg_array(dvstr,tokens,MAX_DV_TOKENS,&itmp);

		if(itmp > 1)
			break;
	}

	dve.type = tokens[0];
	dve.line = tokens[1];
	dve.dialer = tokens[2];
	if(!strcmp(tokens[3],"Any"))
	{
		dve.low_baud = 1;
		dve.high_baud = 100000L;
	}
	else
	{
		dve.low_baud = atoi(tokens[3]);
		if(!(cptr = strchr(tokens[3],'-')))
			dve.high_baud = dve.low_baud;
		else
			dve.high_baud = atoi(cptr + 1);
	}
	dve.dialprog = tokens[4];
	dve.token = tokens[5];
	return(&dve);

}	/* end of getdvent */

/*+-------------------------------------------------------------------------
	getdvbaud(baud) - get Devices entry matching baud rate
--------------------------------------------------------------------------*/
struct dvent *
getdvbaud(baud)
uint baud;
{
struct dvent *dve;

#ifdef CHOOSE_DEBUG
	pprintf("getdvbaud %u\n",baud);
#endif

	while(1)
	{
		if((dve = getdvent()) == (struct dvent *)0)
			return(dve);
		if((dve->low_baud <= baud) && (baud <= dve->high_baud))
		{
#ifdef CHOOSE_DEBUG
			pprintf("returning %s\n",dve->line);
			Nap(2000L);
#endif
			return(dve);
		}
	}
	/*NOTREACHED*/

}	/* end of getdvbaud */

/*+-------------------------------------------------------------------------
	getdvline(line) - get Devices entry matching line
calling argument 'line's is string AFTER "/dev/"
--------------------------------------------------------------------------*/
struct dvent *
getdvline(line)
char *line;
{
struct dvent *dve;

#ifdef CHOOSE_DEBUG
	pprintf("getdvline %s\n",line);
#endif

	while(1)
	{
		if((dve = getdvent()) == (struct dvent *)0)
			return(dve);
		if(!strcmp(dve->line,line))
			return(dve);
	}
	/*NOTREACHED*/

}	/* end of getdvline */

/*+-------------------------------------------------------------------------
	dvtype_match(typespec,dvtype) - match between pde typespec and dvtype?

returns 1 if pde type specification 'typespec' matches Devices device 'type'
--------------------------------------------------------------------------*/
int
dvtype_match(typespec,dvtype)
char *typespec;
char *dvtype;
{
char *emsg;
char *match;
int matchlen;
int exact_required = 1;
char cmpbuf[128];

	if(*typespec == '=')
		typespec++;
	else if(*typespec == '/')
	{
		exact_required = 0;
		typespec++;
	}

	if(exact_required)
	{
		if(!strcmp(dvtype,typespec))
			return(1);
	}
	else
	{
		if(regexp_compile(typespec,cmpbuf,sizeof(cmpbuf),&emsg))
			return(0);
		if(regexp_scan(cmpbuf,dvtype,&match,&matchlen))
			return(1);
	}
	return(0);

}	/* end of dvtype_match */

/*+-------------------------------------------------------------------------
	getdvtype(type) - get Devices entry matching type

type is either 'Device_type'   search for exact match on Device_type
               '=Device_type'  search for exact match on Device_type
               '/regexp'       search for match with regular expression

you must make sure any supplied regexp is a valid one, for regexp
compilation errors are indistinguishable from other seach failures

uses optimized implementation of dvtype_match functionality
--------------------------------------------------------------------------*/
struct dvent *
getdvtype(type)
char *type;
{
struct dvent *dve;
char *emsg;
char *match;
int matchlen;
int exact_required = 1;
char cmpbuf[128];

	if(*type == '=')
		type++;
	else if(*type == '/')
	{
		exact_required = 0;
		type++;
		if(regexp_compile(type,cmpbuf,sizeof(cmpbuf),&emsg))
			return((struct dvent *)0);
	}

	while(1)
	{
		if((dve = getdvent()) == (struct dvent *)0)
			return(dve);
		if(exact_required)
		{
			if(!strcmp(dve->type,type))
				return(dve);
		}
		else
		{
			if(regexp_scan(cmpbuf,dve->type,&match,&matchlen))
				return(dve);
		}
	}
	/*NOTREACHED*/

}	/* end of getdvtype */

/*+-------------------------------------------------------------------------
	enddvent()
--------------------------------------------------------------------------*/
void
enddvent()
{
	if(fpdv != NULL)
	{
		fclose(fpdv);
		fpdv = NULL;
	}
}	/* end of enddvent */

/*+-------------------------------------------------------------------------
	dialstr_translate(translate_list,to_translate) - translate dial strings
--------------------------------------------------------------------------*/
void
dialstr_translate(translate_list,to_translate)
register char *translate_list;
char *to_translate;
{
	register char *cptr;

	while(*translate_list && *(translate_list + 1))
	{
		for(cptr=to_translate; *cptr; cptr++)
		{
			if(*translate_list == *cptr)
				*cptr = *(translate_list + 1);
		}
		translate_list += 2;
	}
}	/* end of dialstr_translate */

/*+-------------------------------------------------------------------------
	getdlent() - get first or next Dialers entry (a la getpwent)
--------------------------------------------------------------------------*/
struct dlent *
getdlent()
{
int itmp;
#define MAX_DL_TOKENS 3
char *tokens[MAX_DL_TOKENS];
static struct dlent dle;
static char dlstr[128];
char *strchr();

	if(!there_is_hdb_on_this_machine)
		return((struct dlent *)0);

	if(fpdl == NULL)
	{
		if((fpdl = fopen(Dialers_file,"r")) == NULL)
		{
			pperror(Dialers_file);
			return((struct dlent *)0);
		}
	}

	while(1)
	{
		if(fgets(dlstr,sizeof(dlstr),fpdl) == NULL)
			return((struct dlent *)0);
		if(((itmp = strlen(dlstr)) > 0) && (dlstr[itmp - 1] == '\n'))
			dlstr[--itmp] = 0;
		if((dlstr[0] == '#') || (dlstr[0] == ' ') || (!itmp))
			continue;
		for(itmp = 0; itmp < MAX_DL_TOKENS; itmp++)
			tokens[itmp] = "";
		if(tokens[0] = arg_token(dlstr," \t\r\n"))
		{
			if(tokens[1] = arg_token((char *)0," \t\r\n"))
			{
			extern char *str_token_static;
				tokens[2] = skip_ld_break(str_token_static);
			}
		}
		break;
	}

	dle.name = tokens[0];
	dle.tlate = tokens[1];
	dle.script = tokens[2];
	return(&dle);

}	/* end of getdlent */

/*+-------------------------------------------------------------------------
	enddlent()
--------------------------------------------------------------------------*/
void
enddlent()
{
	if(fpdl != NULL)
	{
		fclose(fpdl);
		fpdl = NULL;
	}
}	/* end of enddlent */

/*+-------------------------------------------------------------------------
	getdlentname(name) - get Dialers entry by name
--------------------------------------------------------------------------*/
struct dlent *
getdlentname(name)
char *name;
{
register struct dlent *tdle;

	while(tdle = getdlent())
	{
		if(!strcmp(name,tdle->name))
			break;
	}
	return(tdle);

}	/* end of getdlentname */

/*+-------------------------------------------------------------------------
	choose_line(baud) - user is dialing a remote and will take 'Any' line
--------------------------------------------------------------------------*/
void
choose_line(baud)
uint baud;
{
struct dvent *dve = (struct dvent *)0;
char tentative[sizeof(shm->Lline)];
int itmp;

#ifdef CHOOSE_DEBUG
	pprintf("\nchoose_line baud=%u current line=\n",baud,shm->Lline);
#endif

	if(shm->Lline[0])
	{
		dve = getdvline(shm->Lline + 5);
		enddvent();
	}

/* if shm->Lline is open and baud rate ok, no further */
	if(shm->Liofd > 0)
	{
		if(dve && ((dve->low_baud <= baud) && (baud <= dve->high_baud)))
			return;
		lclose();	/* won't do ... gotta find a new one */
		goto GET_LINE_THAT_MATCHES_BAUD;
	}

	if(!shm->Lline[0])
		goto GET_LINE_THAT_MATCHES_BAUD;

/*
 * check sanity of current line
 */
	if(access(shm->Lline,6))
		goto GET_LINE_THAT_MATCHES_BAUD;
	dve = getdvline(shm->Lline + 5);
	enddvent();
	if(!dve)	/* if no match in Devices, let ecu fend for itself */
		goto GET_LINE_THAT_MATCHES_BAUD;

/*
 * see if shm->Lline in use by someone else; if not and baud rate ok, no further
 */
	switch(utmp_status(shm->Lline))
	{
		case US_NOTFOUND:	/* not in utmp, or getty dead */
			if((itmp = line_locked(shm->Lline)) && (itmp != LOPEN_WE_GOT_IT))
				goto GET_LINE_THAT_MATCHES_BAUD;
		case US_LOGIN:		/* enabled for login, idle */
		case US_WEGOTIT:	/* we already have the line */
			if((dve->low_baud <= baud) && (baud <= dve->high_baud))
				return;
	}

/* we've got to pick a new line */
GET_LINE_THAT_MATCHES_BAUD:

	strcpy(tentative,"/dev/");
	while(1)
	{
		if(!(dve = getdvbaud(baud)))
		{
			enddvent();		/* no match ... */
			return;			/* ... so let current shm->Lline fail */
		}
/* by now, we know shm->Lline wont work */
		if(!strcmp(dve->line,shm->Lline + 5))
			continue;

/* if not acu, dont use it */
		if(ulindex(dve->type,"ACU") < 0)
			continue;

		strncat(tentative,dve->line,sizeof(shm->Lline) - 5 - 1);
		tentative[sizeof(shm->Lline) - 1] = 0;
		switch(utmp_status(tentative))
		{
			case US_NOTFOUND:	/* not in utmp, or getty dead */
				if(itmp = line_locked(tentative))
					continue;
			case US_WEGOTIT:
#if defined(SHARE_DEBUG)
				ecu_log_event(getpid(),"looking for line found we own it");
#endif
			case US_LOGIN:		/* enabled for login, idle */
				strcpy(shm->Lline,tentative);	/* @@@@@@@@ MATCH @@@@@@@@@ */
#if defined(M_SYSV)
				itmp = strlen(shm->Lline) - 1;
				shm->Lline[itmp] = to_lower(shm->Lline[itmp]);
#endif
				enddvent();	
				return;
		}
	}
	/*NOTREACHED*/

}	/* end of choose_line */

/*+-------------------------------------------------------------------------
	choose_device(type,baud) - user wants line with 'type' and 'baud'

return 0 if line chosen, -1 if failed to find a line
--------------------------------------------------------------------------*/
int
choose_device(type,baud)
char *type;
uint baud;
{
struct dvent *dve = (struct dvent *)0;
char s32[32];
int utmpstat;
#if defined(M_SYSV)	/* SCO device naming */
char *sptr;
#endif

#ifdef CHOOSE_DEBUG
	pprintf("\nchoose_device type='%s' baud=%u curr line='%s'\n",
		type,baud,shm->Lline);
#endif

	if(shm->Lline[0])
	{
		dve = getdvline(shm->Lline + 5);
		enddvent();
	}

/* if shm->Lline is open and ok, no further */
	if(shm->Liofd > 0)
	{
		if(dve && dvtype_match(type,dve->type) &&
			(dve->low_baud <= baud) && (baud <= dve->high_baud))
		{
#ifdef CHOOSE_DEBUG
			pprintf("currently open line ok\n");
#endif
			return(0);
		}
#ifdef CHOOSE_DEBUG
		pprintf("closing currently open line\n");
		if(dve)
		{
			pprintf("dve type=%s lo=%u hi=%d\n",
				dve->type,dve->low_baud,dve->high_baud);
		}
#endif
		lclose();	/* won't do ... gotta find a new one */
		goto GET_LINE_THAT_MATCHES_TYPE;
	}

/*
 * check sanity of current line
 */
	if(!shm->Lline[0])
		goto GET_LINE_THAT_MATCHES_TYPE;

	if(access(shm->Lline,6))
		goto GET_LINE_THAT_MATCHES_TYPE;

	if(!dve)
		goto GET_LINE_THAT_MATCHES_TYPE;

#ifdef CHOOSE_DEBUG
	pprintf("%s appears valid\n",shm->Lline);
#endif

/*
 * see if shm->Lline in use by someone else; if not and baud rate ok, no further
 */
	switch(utmp_status(shm->Lline))
	{
		case US_WEGOTIT:
#ifdef CHOOSE_DEBUG
			pprintf("we own the line\n");
#endif
			return(0);
		case US_NOTFOUND:	/* not in utmp, or getty dead */
			if(line_locked(s32))
				goto GET_LINE_THAT_MATCHES_TYPE;
		case US_LOGIN:		/* enabled for login, idle */
			if((dve->low_baud <= baud) && (baud <= dve->high_baud))
			{
#ifdef CHOOSE_DEBUG
				pprintf("current line login idle\n");
#endif
				return(0);
			}
	}

/* we've got to pick a new line */
GET_LINE_THAT_MATCHES_TYPE:
#ifdef CHOOSE_DEBUG
	pprintf("searching for line\n");
#endif
	strcpy(s32,"/dev/tty");
	while(1)
	{
		if(!(dve = getdvtype(type)))
		{
#ifdef CHOOSE_DEBUG
			pprintf("no line matches type '%s'\n",type);
#endif
			enddvent();		/* no match ... */
			return(1);
		}
		strcpy(s32 + 8,dve->line + 3);	/* yeech ... make "/dev/ttyxx" */
		utmpstat = utmp_status(s32);
		switch(utmpstat)
		{
			case US_NOTFOUND:	/* not in utmp, or getty dead */
				if(line_locked(s32))
					continue;
				/* fall through */

			case US_WEGOTIT:	/* we own the line */
#if defined(LOG_HDBDIAL)
			pprintf("we own the line we just passed on using!\n");
#endif
			case US_LOGIN:		/* enabled for login, idle */
				if((dve->low_baud > baud) || (baud > dve->high_baud))
					continue;
				strcpy(shm->Lline,s32);	/* @@@@@@@@@@@ MATCH @@@@@@@@@ */

#if defined(M_SYSV)	/* SCO device naming */
				sptr = shm->Lline + strlen(shm->Lline) - 1;
				if(isalpha(*sptr))
					*sptr = to_lower(*sptr);
#endif
#ifdef CHOOSE_DEBUG
				pprintf("line %s matches type '%s' (utmpstat=%u)\n",
					shm->Lline,type,utmpstat);
#endif
				enddvent();	
				return(0);
		}
	}
	/*NOTREACHED*/

}	/* end of choose_device */

/*+-------------------------------------------------------------------------
	hdb_dial_error(errcode)

also sets iv[0] to dial command status
--------------------------------------------------------------------------*/
char *
hdb_dial_error_text(errcode)
int errcode;
{

	iv[0] = 1;
	switch(errcode & 0x7F)
	{
		case RCE_INUSE:
			return("!Line in use");
		case RCE_SIG:
			iv[0] = 2;
			return("!Interrupted");
		case RCE_ARGS:
			return("!Invalid arguments");
		case RCE_PHNO:
			return("!Invalid phone number");
		case RCE_SPEED:
			return("!Bad baud rate");
		case RCE_OPEN:
			return("!Line open error");
		case RCE_IOCTL:
			return("!Ioctl error");
		case RCE_TIMOUT:
			iv[0] = 3;
			return("!Modem Error");
		case RCE_NOTONE:
			return("NO DIAL TONE");
		case RCE_BUSY:
			return("BUSY");
		case RCE_NOCARR:
			return("NO CARRIER");
		case RCE_ANSWER:
			return("NO ANSWER");
		default:
			
		case RCE_NULL:
			return("unknown error\n");
	}
}	/* end of hdb_dial_error */

/*+-------------------------------------------------------------------------
	hdb_dial(presult) - dial with uucp dialer if we can

return 0 if connected
       1 if dial failed
       2 if interrupted
       3 if modem error
       4 if use ecu DCE dialer
--------------------------------------------------------------------------*/
int
hdb_dial(presult)
char **presult;
{
int itmp;
int dial_pid;
int wait_status;
int old_ttymode = get_ttymode();
SIGTYPE (*original_sighdlr)();
struct dvent *dve;
struct dlent *dle = (struct dlent *)0;
char baudstr[10];
char dbgstr[10];
char dial_log[100];
char *stripped_num;
char *sptr;
char *dptr;
static char stat_s20[20];
ulong colors_at_entry = colors_current;
extern char *make_char_graphic();
char token[128];        /* translated dialer token */
#ifdef WHT	/* hack for immediate need ... make a real feature later */
FILE *fp;
char credit_file[128];
#endif

	if(sigint)	/* don't even start if console interrupt posted */
	{
		sigint = 0;
		return(2);
	}

	if(!there_is_hdb_on_this_machine)
		return(4);

	strcpy(dial_log,shm->Lline + 5);

#if defined(M_SYSV)
	sptr = dial_log + strlen(dial_log) - 1;
	if(isalpha(*sptr))
		*sptr = to_upper(*sptr);
#endif

	dve = getdvline(dial_log);
	enddvent();
	if(!dve)
	{
		pputs("no Devices entry for line ... using ecu dialer\n");
		return(4);
	}

	dial_log[0] = 0;
	if(*dve->dialprog != '/')
	{
		dle = getdlentname(dve->dialprog);
		enddlent();
		if(!dle)
		{
			sprintf(dial_log,
				"UUCPDIAL Devices entry %s: '%s' not found in Dialers",
				shm->Lline + 5,dve->dialprog);
		}
	}
	else if(access(dve->dialprog,1))
	{
		sprintf(dial_log,"UUCPDIAL Devices entry %s: (%s) %s",
			shm->Lline + 5,dve->dialprog,sys_errlist[errno]);
	}

	if(dial_log[0])
	{
#if defined(LOG_HDBDIAL)
		ecu_log_event(getpid(),dial_log);
#endif
		pputs(dial_log + 9);
		pputs("\ntrying ecu dialer\n");
		return(4);
	}

	stripped_num = strip_phone_num(shm->Ltelno);

#if defined(WHT) || defined(TELNO_DOLLAR_CREDIT_CARD)
	/* hack for immediate need ... make a real feature later */
	dptr = stripped_num;
	if(*(dptr - 1) == '$')
	{
		*--dptr = 0;
		get_home_dir(credit_file);
		strcat(credit_file,"/.ecu/.credit");
		chmod(credit_file,0400);	/* let's keep this one quiet */
		if(fp = fopen(credit_file,"r"))
		{
			*dptr++ = ',';
			*dptr++ = ',';
			*dptr++ = ',';
			*dptr++ = ',';
			*dptr = 0;
			fgets(dptr,30,fp);
			fclose(fp);
		}
		if(!fp || !(*dptr))
		{
			pprintf("credit card error\n");
			iv[0] = 1;
			return(1);
		}
		if(*(dptr + strlen(dptr) - 1) == 0x0A)
			*(dptr + strlen(dptr) - 1) = 0; /* kill NL */
	}
#endif /* WHT */

  /* Translate Token now (thanks to ache@hq.demos.su) */

	if (dve->token == (char *)0 || !dve->token[0])
		strcpy(token, stripped_num);
	else {
		dptr = token;
		for (sptr = dve->token; *sptr; sptr++)
			if (*sptr != '\\')
				*dptr++ = *sptr;
			else {
				char *s, *t;

				switch (*(sptr + 1)) {

				/* Direct */
				case 'D':
					sptr++;
					s = stripped_num;
				Dir:
					while (*s)
						*dptr++ = *s++;
					break;

				/* Dialcodes Translate */
				case 'T':
					sptr++;
					s = stripped_num;
					t = dialcodes_translate(&s);
					while(*t)
						*dptr++ = *t++;
					goto Dir;

				default:
					*dptr++ = '\\';
					break;
				}
			}
		*dptr = 0;
	}

	pprintf("Type %s to abort ... ",
		(kbdintr == 0x7F) ? "DEL" : make_char_graphic(kbdintr,0));

	if(Ldial_debug_level)
	{
		ttymode(0);
		pputs("\n");
	}
	else
		ttymode(2);

	if(!dle)
	{
		if(access(dve->dialprog,1))
		{
			pperror(dve->dialprog);
			pputs("trying ecu dialer\n");
			return(4);
		}
		sprintf(baudstr,"%u",shm->Lbaud);
		sprintf(dbgstr,"-x%d",Ldial_debug_level);
		original_sighdlr = signal(SIGCLD,SIG_DFL);
		if((dial_pid = smart_fork()) == 0)
		{
			signal(SIGINT,SIG_DFL);
			execl(dve->dialprog,
#if defined(WHT) || defined(ECUdial)
				"ECUdial",
#else
				dve->dialprog,
#endif
				dbgstr, shm->Lline,token,baudstr,(char *)0);
			exit(-1);
		}

		wait_status = (RC_FAIL | RCE_SIG) << 8;
		while(((itmp = wait(&wait_status)) != dial_pid) && (itmp != -1))
			;
		signal(SIGCLD,original_sighdlr);
		ttymode(old_ttymode);
		ttyflush(1);

		if(sigint)	/* keyboard interrupt? */
		{
			kill(dial_pid,9);	/* kill dialer */
			ltoggle_dtr();		/* drop line */
			sigint = 0;			/* reset SIGINT indication */
		}
		lreset_ksr(); /* uucp dialers are nice guys, but lets use our termio */

#if defined(LOG_HDBDIAL)
		if(wait_status)
		{
			sprintf(dial_log,"UUCPDIAL %s %s 0x%04x",
				dve->dialprog,token,wait_status);
			ecu_log_event(getpid(),dial_log);
		}
#endif

		/*
		 * if system reports interrupt, fake dial-reported status
		 */
		if(wait_status & 0xFF)
			wait_status = (RC_FAIL | RCE_SIG) << 8;

		if((wait_status & 0xFF00) == 0xFF00)
		{
			pputs("uucp dial failure (see ~/.ecu/log) ... trying ecu dialer\n");
			return(4);
		}

		wait_status = (wait_status >> 8) & 0xFF;

		if(!(wait_status & ~RC_BAUD))
		{
			char *cptr;

			wait_status &= RC_BAUD;
			switch (wait_status) 
			{
			case 0:         cptr = baudstr; break;  /* SAME */
			case B50:       cptr = "50"; break;
			case B75:       cptr = "75"; break;
			case B110:      cptr = "110"; break;
			case B134:      cptr = "134.5"; break;
			case B150:      cptr = "150"; break;
			case B200:      cptr = "200"; break;
			case B300:      cptr = "300"; break;
			case B600:      cptr = "600"; break;
			case B1200:     cptr = "1200"; break;
			case B1800:     cptr = "1800"; break;
			case B2400:     cptr = "2400"; break;
			case B4800:     cptr = "4800"; break;
			case B9600:     cptr = "9600"; break;
#if defined(B19200)
			case B19200:    cptr = "19200"; break;
#endif
#if defined(B38400)
			case B38400:    cptr = "38400"; break;
#endif
			default:
				switch (wait_status) {
				case EXTA:      cptr = "EXTA"; break;
				case EXTB:      cptr = "EXTB"; break;
				default:        cptr = "????"; break;
				}
			}

			sprintf(stat_s20,"CONNECT %s",cptr);
			*presult = stat_s20;    /* DCE_dial will report result code */
			return(0);
		}

		*presult = hdb_dial_error_text(wait_status);
		setcolor(colors_error);
		pputs(*presult);
		setcolor(colors_at_entry);
		pputc('\n');
		ltoggle_dtr();
	}
	else
	{
		extern int expresp_verbosity;
		pprintf("using Dialers entry '%s'\n",dle->name);
		expresp_verbosity = (proc_level & proctrace) ? 1 : 0;
		if(execute_expresp(dle->script))
		{
			*presult = "DIALER SCRIPT FAILED";
			setcolor(colors_error);
			pputs(*presult);
			setcolor(colors_at_entry);
			pputc('\n');
			iv[0] = 1;
		}
		else
		{
			extern char last_Speed_result[];
			if(last_Speed_result[0])
				*presult = last_Speed_result;
			else
			{
				sprintf(stat_s20,"CONNECT %u",shm->Lbaud);
				*presult = stat_s20;	/* DCE_dial will report result code */
			}
			setcolor(colors_at_entry);
			iv[0] = 0;
		}
	}

	return((int)iv[0]);

}	/* end of hdb_dial */

/*+-------------------------------------------------------------------------
	hdb_init() - initialize HoneyDanBerInterface
--------------------------------------------------------------------------*/
void
hdb_init()
{
char *hdbfiles = HDBFILES;		/* system independent location */
int buflen = strlen(hdbfiles) + 64;
char *emsg = "hdb_init memory allocation failed!\n";

	if(!(Devices_file = malloc(buflen)))
	{
		pputs(emsg);
		termecu(TERMECU_MALLOC);
	}
	strcpy(Devices_file,hdbfiles);
	strcat(Devices_file,"/Devices");

	if(!(Dialers_file = malloc(buflen)))
	{
		pputs(emsg);
		termecu(TERMECU_MALLOC);
	}
	strcpy(Dialers_file,hdbfiles);
	strcat(Dialers_file,"/Dialers");

	if(!(Dialcodes_file = malloc(buflen)))
	{
		pputs(emsg);
		termecu(TERMECU_MALLOC);
	}
	strcpy(Dialcodes_file,hdbfiles);
	strcat(Dialcodes_file,"/Dialcodes");

	there_is_hdb_on_this_machine = !access(Devices_file,4);

}	/* end of hdb_init */

/*+-------------------------------------------------------------------------
dialcodes_translate(phone)  -  translate first part of phone using Dialcodes
--------------------------------------------------------------------------*/
char *
dialcodes_translate(phone)
char **phone;
{
FILE *f;
int itmp;
#define MAX_DLC_TOKENS 2
char *tokens[MAX_DLC_TOKENS];
static char dlstr[128];

	if ((f = fopen(Dialcodes_file, "r")) == NULL)
		return "";

	while(1) {
		if(fgets(dlstr,sizeof(dlstr),f) == NULL)
			break;
		if(((itmp = strlen(dlstr)) > 0) && (dlstr[itmp - 1] == '\n'))
			dlstr[--itmp] = 0;
		if((dlstr[0] == '#') || (dlstr[0] == ' ') || (!itmp))
			continue;
		if(tokens[0] = arg_token(dlstr," \t\r\n"))
		{
			if (!(tokens[1] = arg_token((char *)0," \t\r\n")))
				tokens[1] = "";
			itmp = strlen(tokens[0]);
			if (strncmp(*phone, tokens[0], itmp))
				continue;
			fclose(f);
			*phone += itmp;
			return tokens[1];
		}
		break;
	}

	fclose(f);
	return "";
}

/*+-------------------------------------------------------------------------
      strip_phone_num(sptr)   -  remove junk characters from phone
--------------------------------------------------------------------------*/
char *
strip_phone_num(sptr)
char *sptr;
{
static char stripped_num[64];
char *dptr;

	dptr = stripped_num;
	while(*sptr)
	{
		if((*sptr == '(') || (*sptr == ')')
#if defined(WHT) || defined(STRIP_TELNO_HYPHENS)
			|| (*sptr == '-')	/* some want '-' for pauses; I use ',' */
#endif
			)
		{
			sptr++;
			continue;
		}
		*dptr++ = *sptr++;
	}
	*dptr = 0;

	return stripped_num;
}

/* vi: set tabstop=4 shiftwidth=4: */
/* end of hdbintf.c */
